@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2013, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -65,6 +65,8 @@ struct mod_action;
 
 typedef struct {
 	int ioprio;
+	int iolimit;
+	int iopslimit;
 } io_param;
 
 typedef struct name_param {
@@ -90,7 +92,7 @@ typedef struct vps_res {
 	net_param net;		/**< network parameters. */
 	cpu_param cpu;		/**< cpu parameters. */
 	dev_param dev;		/**< device parameters. */
-	ub_param ub;		/**< UBC aprameters. */
+	ub_param ub;		/**< UBC parameters. */
 	cap_param cap;		/**< capability parameters. */
 	dq_param dq;		/**< disk quota parameters. */
 	cpt_param cpt;		/**< chekpointing parameters */
@@ -166,13 +168,15 @@ typedef struct vps_param vps_param;
  * @param veid		CT ID.
  * @param actions	distribution action scripts.
  * @param fs		CT FS parameters
+ * @param ub		CT UBC parameters (if NULL taken from param)
  * @param param		CT parameters.
  * @param vps_state	CT state.
  * @param action	external modules.
  * @return		0 on success.
  */
 int vps_setup_res(vps_handler *h, envid_t veid, dist_actions *actions,
-	fs_param *fs, vps_param *param, int vps_state, skipFlags skip,
+	fs_param *fs, ub_param *ub, vps_param *param,
+	int vps_state, skipFlags skip,
 	struct mod_action *action);
 int need_configure(vps_res *res);
 
