@@ -262,7 +262,10 @@ int exec_container_init(struct arg_start *arg,
 		}
 	}
 
-	mount("proc", "/proc", "proc", 0, 0);
+	if (access("/proc", F_OK) == 0 && mount("proc", "/proc", "proc", 0, 0))
+		return vzctl_err(VZ_SYSTEM_ERROR, errno,
+				"Failed to mount /proc");
+
 	if (stat_file("/sys") == 1)
 		mount("sysfs", "/sys", "sysfs", 0, 0);
 
@@ -621,6 +624,9 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 			quota_inc(&res->dq, -100);
 	}
 
+	if ((ret = fill_vswap_ub(&res->ub, &res->ub)))
+		return ret;
+
 	if (pipe(wait_p) < 0) {
 		logger(-1, errno, "Can not create pipe");
 		return VZ_RESOURCE_ERROR;
@@ -660,7 +666,7 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 	if (ret)
 		goto err;
 
-	if ((ret = vps_setup_res(h, veid, &actions, &res->fs, param,
+	if ((ret = vps_setup_res(h, veid, &actions, &res->fs, NULL, param,
 		STATE_STARTING, skip, mod)))
 	{
 		goto err;
