@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2013, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -89,7 +89,8 @@ static int vps_2quota_perm(vps_handler *h, int veid, dev_t device)
 }
 
 int vps_setup_res(vps_handler *h, envid_t veid, dist_actions *actions,
-	fs_param *fs, vps_param *param, int vps_state, skipFlags skip,
+	fs_param *fs, ub_param *ub, vps_param *param,
+	int vps_state, skipFlags skip,
 	struct mod_action *action)
 {
 	int ret;
@@ -98,7 +99,7 @@ int vps_setup_res(vps_handler *h, envid_t veid, dist_actions *actions,
 	if (skip & SKIP_SETUP)
 		return 0;
 	if (vps_state != STATE_STARTING) {
-		if ((ret = vps_set_ublimit(h, veid, &res->ub)))
+		if ((ret = vps_set_ublimit(h, veid, ub ? : &res->ub)))
 			return ret;
 	}
 	if ((ret = vps_net_ctl(h, veid, DEL, &param->del_res.net, actions,
@@ -126,7 +127,7 @@ int vps_setup_res(vps_handler *h, envid_t veid, dist_actions *actions,
 		return ret;
 	if ((ret = vps_meminfo_set(h, veid, &res->meminfo, param, vps_state)))
 		return ret;
-	if ((ret = ve_ioprio_set(h, veid, &res->io)))
+	if ((ret = vps_set_io(h, veid, &res->io)))
 		return ret;
 	/* Setup 2nd-level quota */
 	if (is_2nd_level_quota_on(&res->dq)) {
