@@ -122,28 +122,7 @@ if (ub->name == NULL) {							\
 inline static int is_ub_empty(ub_param *ub)
 {
 #define CHECK_UB(name)	if (ub->name != NULL) return 0;
-
-	CHECK_UB(kmemsize)
-	CHECK_UB(lockedpages)
-	CHECK_UB(privvmpages)
-	CHECK_UB(shmpages)
-	CHECK_UB(numproc)
-	CHECK_UB(physpages)
-	CHECK_UB(vmguarpages)
-	CHECK_UB(oomguarpages)
-	CHECK_UB(numtcpsock)
-	CHECK_UB(numflock)
-	CHECK_UB(numpty)
-	CHECK_UB(numsiginfo)
-	CHECK_UB(tcpsndbuf)
-	CHECK_UB(tcprcvbuf)
-	CHECK_UB(othersockbuf)
-	CHECK_UB(dgramrcvbuf)
-	CHECK_UB(numothersock)
-	CHECK_UB(numfile)
-	CHECK_UB(dcachesize)
-	CHECK_UB(numiptent)
-	CHECK_UB(swappages)
+	FOR_ALL_UBC(CHECK_UB)
 #undef CHECK_UB
 
 	return 1;
@@ -178,14 +157,10 @@ static const char *get_ub_name(unsigned int res_id)
 
 int set_ublimit(vps_handler *h, envid_t veid, ub_param *ub)
 {
-	int vswap = is_vswap_config(ub);
-	const unsigned long unlim_ub[2] = {LONG_MAX, LONG_MAX};
 	const unsigned long *res;
 
 #define SET_UB_LIMIT(name, id)						\
 res = ub->name;								\
-if (vswap && !res)							\
-	res = unlim_ub;							\
 if (res != NULL) {							\
 	if (setublimit(veid, id, res)) {				\
 		logger(-1, errno, "setublimit %s %lu:%lu failed",	\
@@ -259,28 +234,8 @@ void free_ub_param(ub_param *ub)
 #define FREE_P(x) free(ub->x); ub->x = NULL;
 	if (ub == NULL)
 		return;
-	FREE_P(kmemsize)
-	FREE_P(lockedpages)
-	FREE_P(privvmpages)
-	FREE_P(shmpages)
-	FREE_P(numproc)
-	FREE_P(physpages)
-	FREE_P(vmguarpages)
-	FREE_P(oomguarpages)
-	FREE_P(numtcpsock)
-	FREE_P(numflock)
-	FREE_P(numpty)
-	FREE_P(numsiginfo)
-	FREE_P(tcpsndbuf)
-	FREE_P(tcprcvbuf)
-	FREE_P(othersockbuf)
-	FREE_P(dgramrcvbuf)
-	FREE_P(numothersock)
-	FREE_P(numfile)
-	FREE_P(dcachesize)
-	FREE_P(numiptent)
-	FREE_P(avnumproc)
-	FREE_P(swappages)
+	FOR_ALL_UBC(FREE_P)
+	FREE_P(vm_overcommit)
 #undef FREE_P
 }
 
@@ -338,39 +293,50 @@ int add_ub_param(ub_param *ub, ub_res *res)
 	return 0;
 }
 
-void merge_ub(ub_param *dst, ub_param *src)
+/** Set secondary beancounters in vswap mode,
+ *  if they were not set explicitly,
+ *  based on ram, swap and overcommit.
+ *
+ *  @param cfg		UBC parameters from CT config
+ *  @param cmd		UBC parameters from command line
+ */
+int fill_vswap_ub(ub_param *cfg, ub_param *cmd)
 {
-#define MERGE_P2(x)						\
-if ((src->x) != NULL) {						\
-	if ((dst->x) == NULL)					\
-		dst->x = malloc(sizeof(*(dst->x)) * 2);		\
-	dst->x[0] = src->x[0];					\
-	dst->x[1] = src->x[1];					\
-}
+	float ovc = 0;
+	unsigned long ram, swap;
 
-	MERGE_P2(kmemsize)
-	MERGE_P2(lockedpages)
-	MERGE_P2(privvmpages)
-	MERGE_P2(shmpages)
-	MERGE_P2(numproc)
-	MERGE_P2(physpages)
-	MERGE_P2(vmguarpages)
-	MERGE_P2(oomguarpages)
-	MERGE_P2(numtcpsock)
-	MERGE_P2(numflock)
-	MERGE_P2(numpty)
-	MERGE_P2(numsiginfo)
-	MERGE_P2(tcpsndbuf)
-	MERGE_P2(tcprcvbuf)
-	MERGE_P2(othersockbuf)
-	MERGE_P2(dgramrcvbuf)
-	MERGE_P2(numothersock)
-	MERGE_P2(numfile)
-	MERGE_P2(dcachesize)
-	MERGE_P2(numiptent)
-	MERGE_P2(avnumproc)
-	MERGE_P2(swappages)
-#undef MERGE_P2
+#define SET(param, bar, lim) do {					\
+	if (!cmd->param && !cfg->param) {				\
+		cmd->param = vz_malloc(sizeof(unsigned long)*2);	\
+		if (!cmd->param)					\
+			goto enomem;					\
+		cmd->param[0] = (bar);					\
+		cmd->param[1] = (lim);					\
+	} } while (0)
+
+	if (!is_vswap_config(cfg) && !is_vswap_config(cmd))
+		return 0;
+
+	ram = (cmd->physpages ? : cfg->physpages)[1];
+	swap = (cmd->swappages ? : cfg->swappages)[1];
+	if (cmd->vm_overcommit)
+		ovc = *cmd->vm_overcommit;
+	else if (cfg->vm_overcommit)
+		ovc = *cfg->vm_overcommit;
+
+	SET(lockedpages, ram, ram);
+	SET(oomguarpages, ram, LONG_MAX);
+	SET(vmguarpages, ram + swap, LONG_MAX);
+	if (ovc)
+		SET(privvmpages, (ram + swap) * ovc, (ram + swap) * ovc);
+	else
+		SET(privvmpages, LONG_MAX, LONG_MAX);
+#undef SET
+
+	return 0;
+
+enomem:
+	return VZ_RESOURCE_ERROR;
 }
 
 /** Read UBC resources current usage from /proc/user_beancounters
