@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2012, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2013, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -83,6 +83,8 @@ static struct option set_opt[] = {
 	{"numiptent",	required_argument, NULL, PARAM_NUMIPTENT},
 	{"avnumproc",	required_argument, NULL, PARAM_AVNUMPROC},
 	{"swappages",	required_argument, NULL, PARAM_SWAPPAGES},
+	{"vm_overcommit",
+			required_argument, NULL, PARAM_VM_OVERCOMMIT},
 	/*	Capability */
 	{"capability",	required_argument, NULL, PARAM_CAP},
 	/*	Network	*/
@@ -141,6 +143,8 @@ static struct option set_opt[] = {
 	{"name",	required_argument, NULL, PARAM_NAME},
 	{"features",	required_argument, NULL, PARAM_FEATURES},
 	{"ioprio",	required_argument, NULL, PARAM_IOPRIO},
+	{"iolimit",	required_argument, NULL, PARAM_IOLIMIT_MB},
+	{"iopslimit",	required_argument, NULL, PARAM_IOPSLIMIT},
 	{"description",	required_argument, NULL, PARAM_DESCRIPTION},
 	{"bootorder",	required_argument, NULL, PARAM_BOOTORDER},
 
@@ -981,7 +985,7 @@ static int set_ve0(vps_handler *h, vps_param *g_p,
 	ret = vps_set_ublimit(h, 0, ub);
 	if (ret)
 		return ret;
-	if ((ret = ve_ioprio_set(h, 0, &cmd_p->res.io)))
+	if ((ret = vps_set_io(h, 0, &cmd_p->res.io)))
 		return ret;
 	if (cpu != NULL)
 		if ((ret = hn_set_cpu(cpu)))
@@ -1044,8 +1048,15 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 		if (ret != 0)
 			return ret;
 	}
+
+	is_run = h != NULL && vps_is_run(h, veid);
+
 	/* Reset UB parameters from config  */
-	if (cmd_p->opt.reset_ub == YES && h != NULL) {
+	if (cmd_p->opt.reset_ub == YES) {
+		if (!is_run) {
+			logger(-1, 0, "Container is not running");
+			return VZ_VE_NOT_RUNNING;
+		}
 		ret = vps_set_ublimit(h, veid, &vps_p->res.ub);
 		*warn_save = 0;
 		return ret;
@@ -1058,7 +1069,6 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 			return ret;
 		}
 	}
-	is_run = h != NULL && vps_is_run(h, veid);
 	if (is_run) {
 		if (cmd_p->res.fs.private_orig != NULL) {
 			free(cmd_p->res.fs.private_orig);
@@ -1162,11 +1172,18 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 
 	/* Only try to apply parameters on a supported kernel */
 	if (h) {
+		ub_param calc_ub = {};
+
+		merge_ub(&calc_ub, &cmd_p->res.ub);
+		fill_vswap_ub(&g_p->res.ub, &calc_ub);
+
 		/* If CT is not running, call this anyway to get relevant
 		 * errors messages like "Can't set CPU: CT is not running"
 		 */
-		ret = vps_setup_res(h, veid, actions, &g_p->res.fs, cmd_p,
+		ret = vps_setup_res(h, veid, actions,
+				&g_p->res.fs, &calc_ub, cmd_p,
 				STATE_RUNNING, SKIP_NONE, &g_action);
+		free_ub_param(&calc_ub);
 	}
 err:
 	free_dist_actions(actions);
