@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2012, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2013, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -45,6 +45,7 @@
 #include "types.h"
 #include "image.h"
 #include "vzfeatures.h"
+#include "io.h"
 
 static struct Cveinfo *veinfo = NULL;
 static int n_veinfo = 0;
@@ -88,29 +89,6 @@ static inline int get_run_ve(int update)
 #define get_run_ve get_run_ve_proc
 #endif
 
-#define FOR_ALL_UBC(func)	\
-	func(kmemsize)		\
-	func(lockedpages)	\
-	func(privvmpages)	\
-	func(shmpages)		\
-	func(numproc)		\
-	func(physpages)		\
-	func(vmguarpages)	\
-	func(oomguarpages)	\
-	func(numtcpsock)	\
-	func(numflock)		\
-	func(numpty)		\
-	func(numsiginfo)	\
-	func(tcpsndbuf)		\
-	func(tcprcvbuf)		\
-	func(othersockbuf)	\
-	func(dgramrcvbuf)	\
-	func(numothersock)	\
-	func(dcachesize)	\
-	func(numfile)		\
-	func(numiptent)		\
-	func(swappages)
-
 /* Print functions */
 static void print_json_str(const char *s)
 {
@@ -321,20 +299,29 @@ static void print_cpu ## name(struct Cveinfo *p, int index)	\
 PRINT_CPU(limit)
 PRINT_CPU(units)
 
-static void print_ioprio(struct Cveinfo *p, int index)
-{
-	if (fmt_json) {
-		printf("%d", (p->io.ioprio < 0) ? 0 : p->io.ioprio);
-		return;
-	}
-
-	if (p->io.ioprio < 0)
-		p_buf += snprintf(p_buf, e_buf - p_buf, "%3s", "-");
-	else
-		p_buf += snprintf(p_buf, e_buf - p_buf, "%3d",
-			p->io.ioprio);
+#define PRINT_IO(name, fmt, min)				\
+static void print_io ## name(struct Cveinfo *p, int index)	\
+{								\
+	int i = p->io.io ## name;				\
+								\
+	if (fmt_json) {						\
+		printf("%d", (i < 0) ? 0 : i);			\
+		return;						\
+	}							\
+								\
+	if (i < min)						\
+		p_buf += snprintf(p_buf, e_buf - p_buf,		\
+				fmt "s", "-");			\
+	else							\
+		p_buf += snprintf(p_buf, e_buf - p_buf,		\
+				fmt "d", i);			\
 }
 
+PRINT_IO(prio, "%3", 0)
+PRINT_IO(limit, "%10", 1)
+PRINT_IO(pslimit, "%4", 1)
+#undef PRINT_IO
+
 static void print_bool(const char *fmt, int val)
 {
 	if (fmt_json)
@@ -521,6 +508,20 @@ static void print_ ## name(struct Cveinfo *p, int index)		\
 PRINT_DQ(diskspace)
 PRINT_DQ(diskinodes)
 
+static void print_vm_overcommit(struct Cveinfo *p, int index)
+{
+	if (fmt_json) {
+		printf("%g", p->vm_overcommit);
+		return;
+	}
+
+	if (p->vm_overcommit == 0)
+		p_buf += snprintf(p_buf, e_buf - p_buf, "%6s", "-");
+	else
+		p_buf += snprintf(p_buf, e_buf - p_buf, "%6g",
+			p->vm_overcommit);
+}
+
 /* Sort functions */
 
 static inline int check_empty_param(const void *val1, const void *val2)
@@ -607,12 +608,18 @@ static int bootorder_sort_fn(const void *val1, const void *val2)
 	return (*r1 > *r2);
 }
 
-static int ioprio_sort_fn(const void *val1, const void *val2)
-{
-	return ((const struct Cveinfo *)val1)->io.ioprio >
-		((const struct Cveinfo *)val2)->io.ioprio;
+#define IO_SORT_FN(name)						\
+static int io ## name ## _sort_fn(const void *val1, const void *val2)	\
+{									\
+	return ((const struct Cveinfo *)val1)->io.io ## name >		\
+		((const struct Cveinfo *)val2)->io.io ## name;		\
 }
 
+IO_SORT_FN(prio)
+IO_SORT_FN(limit)
+IO_SORT_FN(pslimit)
+#undef IO_SORT_FN
+
 static int cpunum_sort_fn(const void *val1, const void *val2)
 {
 	return ((const struct Cveinfo *)val1)->cpunum >
@@ -707,7 +714,7 @@ static struct Cfield field_names[] =
 {"ip", "IP_ADDR", "%-15s", 0, RES_IP, print_ip, ip_sort_fn},
 {"nameserver", "NAMESERVER", "%-15s", 0, RES_NONE, print_nameserver, nameserver_sort_fn},
 {"searchdomain", "SEARCHDOMAIN", "%-15s", 0, RES_NONE, print_searchdomain, searchdomain_sort_fn},
-{"status", "STATUS", "%-9s", 0, RES_NONE, print_status, status_sort_fn},
+{"status", "STATUS", "%-9s", 0, RES_STATUS, print_status, status_sort_fn},
 /*	UBC	*/
 UBC_FIELD(kmemsize, KMEMSIZE),
 UBC_FIELD(lockedpages, LOCKEDP),
@@ -747,6 +754,8 @@ UBC_FIELD(swappages, SWAPP),
 {"cpus", "CPUS", "%5s", 0, RES_CPUNUM, print_cpunum, cpunum_sort_fn},
 
 {"ioprio", "IOP", "%3s", 0, RES_NONE, print_ioprio, ioprio_sort_fn},
+{"iolimit", "IOLIMIT", "%10s", 0, RES_IO, print_iolimit, iolimit_sort_fn},
+{"iopslimit", "IOPS", "%4s", 0, RES_IO, print_iopslimit, iopslimit_sort_fn},
 
 {"onboot", "ONBOOT", "%6s", 0, RES_NONE, print_onboot, none_sort_fn},
 {"bootorder", "BOOTORDER", "%10s", 0, RES_NONE,
@@ -755,6 +764,8 @@ UBC_FIELD(swappages, SWAPP),
 {"features", "FEATURES", "%-15s", 0, RES_NONE, print_features, none_sort_fn},
 {"vswap", "VSWAP", "%5s", 0, RES_NONE, print_vswap, none_sort_fn},
 {"disabled", "DISABL", "%6s", 0, RES_NONE, print_disabled, none_sort_fn},
+{"vm_overcommit", "VM_OVC", "%6s", 0, RES_NONE,
+	print_vm_overcommit, none_sort_fn},
 };
 
 static void *x_malloc(int size)
@@ -1069,6 +1080,10 @@ FOR_ALL_UBC(MERGE_UBC)
 
 #undef MERGE_UBC
 	}
+
+	if (res->ub.vm_overcommit != NULL)
+		ve->vm_overcommit = *res->ub.vm_overcommit;
+
 	if (ve->ip == NULL && !list_empty(&res->net.ip)) {
 		ve->ip = strdup(list2str(NULL, &res->net.ip));
 	}
@@ -1122,7 +1137,11 @@ FOR_ALL_UBC(MERGE_UBC)
 		ve->bootorder = x_malloc(sizeof(*ve->bootorder));
 		*ve->bootorder = *res->misc.bootorder;
 	}
+
 	ve->io.ioprio = res->io.ioprio;
+	ve->io.iolimit = res->io.iolimit;
+	ve->io.iopslimit = res->io.iopslimit;
+
 	if (ve->cpunum == -1 && res->cpu.vcpus != NULL)
 		ve->cpunum = *res->cpu.vcpus;
 	ve->features_mask  = res->env.features_mask;
@@ -1658,6 +1677,28 @@ static int get_ves_cpu()
 	return 0;
 }
 
+static int update_ves_io(void)
+{
+	int limit, i;
+
+	if ((vzctlfd = open(VZCTLDEV, O_RDWR)) < 0)
+		return 1;
+
+	for (i = 0; i < n_veinfo; i++) {
+		if ((veinfo[i].hide) || (veinfo[i].status != VE_RUNNING))
+			continue;
+
+		if (vzctl_get_iolimit(vzctlfd, veinfo[i].veid, &limit) == 0)
+			veinfo[i].io.iolimit = limit;
+
+		if (vzctl_get_iopslimit(vzctlfd, veinfo[i].veid, &limit) == 0)
+			veinfo[i].io.iopslimit = limit;
+	}
+	close(vzctlfd);
+
+	return 0;
+}
+
 static int get_ve_list()
 {
 	DIR *dp;
@@ -1790,7 +1831,10 @@ static int collect()
 	if (check_param(RES_CPUNUM) && !only_stopped_ve)
 	       get_ves_cpunum();
 	read_ves_param();
-	get_mounted_status();
+	if (check_param(RES_IO))
+		update_ves_io();
+	if (check_param(RES_STATUS))
+		get_mounted_status();
 	get_ves_layout();
 	if (check_param(RES_QUOTA)) {
 		get_run_quota_stat();
